<?php
/*
 Plugin Name: 		CLP - Custom Login Page
 Plugin URI: 		https://wordpress.org/plugins/clp-custom-login-page/
 Description:       Best plugin to customize login and Registration page
 Version:           1.1.0
 Author:            NiteoThemes
 Author URI:        https://www.niteothemes.com
 Text Domain:       clp-custom-login-page
 Domain Path:		/languages
 License:           GPL-2.0+
 License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
*/


if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'CLP_Custom_Login_Page' ) ) :

	/**
	 * Main CMP Coming Soon and Maintenance class.
	 *
	 * @since 1.0.0
	**/ 
	class CLP_Custom_Login_Page {
	
		private static $instance;

		/**
		 * Main CLP_Custom_Login_Page Instance.
		 * @since 1.0.0
		 * @return object
		**/
		public static function instance() {
			if ( ! isset( self::$instance ) && !( self::$instance instanceof CLP_Custom_Login_Page ) ) {
				self::$instance = new CLP_Custom_Login_Page();

			}

			return self::$instance;
		}

		/**
         * Class Constructor
		 * @since 1.0.0
		**/
		public function __construct() {
			$this->autoloader();
			$this->constants();
			$this->load_textdomain();
			$this->hooks();
		  }

		/**
         * Define plugin constants
		 * @since 1.0.0
		**/
		private function constants() {
			$this->define( 'CLP_VERSION', '1.1.0' );
			$this->define( 'CLP_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
			$this->define( 'CLP_PLUGIN_PATH', plugin_dir_url( __FILE__ ) );
		}

		/**
		 * Includes required classes files
		 * @since 1.0.0
		**/
		public function autoloader() {
			require_once 'includes/class-clp-autoloader.php';
		}

		/**
		 * Define constant if not already set.
		 * @since 1.0.0
		 * @param  string|string $name Name of the definition.
		 * @param  string|bool   $value Default value.
		**/
		private function define( $name, $value ) {
			if ( ! defined( $name ) ) {
				define( $name, $value );
			}
        }
        
		/**
		 * Load text domain
         * @since 1.0.0
		**/
		public function load_textdomain() {
			load_plugin_textdomain( 'clp-custom-login-page', false, CLP_PLUGIN_DIR . 'languages/' );
		}

		/**
		 * Inits and hooks
		 * @since 1.0.0
		**/
		public function hooks() {
			add_action( 'init', array( $this, 'customize_login_template' ) );
			add_action( 'admin_init', array( $this, 'redirect_customizer' ) );
			add_action( 'admin_menu', array( $this, 'add_menu_items' ) );
			add_action( 'customize_register', array( $this, 'load_customizer' ) );

			add_filter( 'template_include', array( $this, 'change_customizer_template' ), 99 );
			
			add_action( 'wp_ajax_clp_wp_get_attachment_url_ajax', array( 'CLP_Helper_Functions', 'clp_wp_get_attachment_url_ajax' ) );
			add_action( 'wp_ajax_clp_get_unsplash', array( 'CLP_Unsplash_Api', 'clp_get_unsplash' ) );
			
			// add plugin action links
			add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), array( $this,'add_action_links' ) );
		}

		/**
		 * load Customizer
		 * @since 1.0.0
		**/
		public function load_customizer( $wp_customize ) {
			new CLP_Customizer( $this, $wp_customize );
		}

		/**
		 * Add plugin menu
		 * @since 1.0.0
		**/
		public function add_menu_items() {
			$page = add_menu_page(__('Custom Login', 'clp-custom-login-page'), __('Custom Login', 'clp-custom-login-page'), 'manage_options', 'clp-customize', array($this, 'settings_page'), 'dashicons-buddicons-topics' );
		}

		/**
		 * Add settings page
		 * @since 1.0.0
		**/
		public function settings_page() {

		}

		/**
		 * Add Plugin action links
		 * @since 1.0.0
		**/
		public function add_action_links( $links ) {
			 $settings = array(
			 	'<a href="' . admin_url( 'customize.php?autofocus[panel]=clp_panel' ) . '">'.__('Customize Login Page', 'clp-custom-login-page').'</a>',
			 );
			return array_merge( $settings, $links );
		}

		/**
		 * Redirect to Customizer from admin menu 
		 * @since 1.0.0
		**/
		public function redirect_customizer() {
			if ( ! empty( $_GET['page'] ) ) { 
				if ( 'clp-customize' === $_GET['page'] ) { 
					$url = add_query_arg( array( 'autofocus[panel]' => 'clp_panel' ), admin_url( 'customize.php' ) );
					wp_safe_redirect( $url );
				}
			}
		}

		/**
		 * Change to login template if customizer panel is expanded and change the preview template per it's settings
		 * @since 1.0.0
		 * @return string
		**/
		public function change_customizer_template( $template ) {

			if ( is_customize_preview() && isset( $_REQUEST['clp-customize'] ) && is_user_logged_in() ) {
				$new_template = CLP_PLUGIN_DIR . 'includes/customizer-login-template.php';
				return $new_template;
			}

			return $template;
		}

		/**
		 * Include login template customizations to login template to 
		 * @since 1.0.0
		 * @return string
		**/
		public function customize_login_template() {
			if ( stripos( $_SERVER["SCRIPT_NAME"], strrchr( wp_login_url(), '/' ) ) !== false ){
				include_once( CLP_PLUGIN_DIR . 'includes/template-customization.php' );
			}
		}

	}
	
endif;

/*
 * @since 1.0.0
 * @return object|CMP_Coming_Soon_and_Maintenance instance.
 */
function clp_custom_login() {
	return CLP_Custom_Login_Page::instance();
}

// Get the things running
clp_custom_login();