<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
};

class CLP_Customizer_Settings {

    /**
     * Returns settings fields
     * @since 1.0.0
     * @return array
    **/
    public function get_settings_fields() {
        return $this->settings_fields();
    }

    /**
     * Returns CSS fields from settings array for CSS generating
     * @since 1.0.0
    **/
    public function get_css_fields() {

        $settings = $this->settings_fields();

        // $this->grab_theme_settings();

        $css = array();

        foreach ( $settings as $section => $controls ) {


            foreach ( $controls['fields'] as $key => $value ) {
                if ( isset($value['css']) ) {
                    $value['css']['id'] = $value['id'];
                    $value['css']['default'] = $value['default'];

                    if ( isset($value['css']['dependency']) ) {
                        $value['css']['dependency']['default'] = $this->get_css_dependency_default_value( $value['css']['dependency'][0], $section );
                    }

                    array_push( $css, $value['css']);
                }
            }
        }

        return $css;

    }

    /**
     * Helper to get CSS dependency on another settings
     * @since 1.0.0
    **/
    private function get_css_dependency_default_value( $control_id, $section) {

        $settings = $this->settings_fields();
        
        foreach ( $settings[$section]['fields'] as $control ) {
            if ( $control['id'] === $control_id ) {
                return $control['default'];
            }
        }

        return false;
    }

    /**
     * Returns array with default settings
     * @since 1.0.0
     * @return array
    **/
    private function get_default_layout_settings() {

        return array(
            'clp_logo' => 'image',
            'clp_logo-text-color' => '#0073aa',
            'clp_logo-image' => '',
            'clp_logo-image-width' => '80',
            'clp_logo-spacing-top' => '0',
            'clp_logo-spacing-bottom' => '30',
            'clp_logo-contained' => false,
            'clp_logo-google_fonts' => '{"family":"Roboto","variants":["100","100italic","300","300italic","regular","italic","500","500italic","700","700italic","900","900italic"],"selected":{"variant":"regular"}}',
            'clp_logo-text-font_size' => '35',
            'clp_logo-text-letter_spacing' => '0',
            'clp_background' => 'color',
            'clp_background-color' => '#f1f1f1',
            'clp_background-gradient-color1' => '#f1f1f1',
            'clp_background-gradient-color1-position' => '0',
            'clp_background-gradient-color2' => '#878787',
            'clp_background-gradient-color2-position' => '100',
            'clp_background-gradient-angle' => '30',
            'clp_background-pattern' => 'fabric',
            'clp_background-pattern-custom' => '',
            'clp_background-image' => '',
            'clp_background-video_local' => '',
            'clp_background-video_yt' => '',
            'clp_background-video_loop' => '1',
            'clp_background-video_thumb' => '',
            'clp_background-overlay-enable' => false,
            'clp_background-overlay-color' => 'rgba(0,0,0,0.4)',
            'clp_background-blur' => '0',
            'clp_form-width' => '340',
            'clp_form-height' => '380',
            'clp_form-padding' => '20',
            'clp_form-background' => 'rgba(255,255,255,1)',
            'clp_form-blur' => '0',
            'clp_form-border_radius' => '0',
            'clp_form_typography-google_fonts' => '{"family":"Roboto","variants":["100","100italic","300","300italic","regular","italic","500","500italic","700","700italic","900","900italic"],"selected":{"variant":"regular"}}',
            'clp_form-borders' => false,
            'clp_form-border_color' => '#7e8993',
            'clp_form-border_width' => '1',
            'clp_form-shadow' => '1',
            'clp_form-shadow-horizontal_length' => '10',
            'clp_form-shadow-vertical_length' => '10',
            'clp_form-shadow-blur_radius' => '40',
            'clp_form-shadow-spread_radius' => '-10',
            'clp_form-shadow-color' => 'rgba(0,0,0,0.75)',
            'clp_input-background_color' => '#ffffff',
            'clp_input-background_color_focus' => '#ffffff',
            'clp_input-color' => '#32373c',
            'clp_input-color_focus' => '#32373c',
            'clp_input-label_color' => '#32373c',
            'clp_input-font_size_label' => '14',
            'clp_input-font_size_input' => '24',
            'clp_input-height' => '40',
            'clp_input-border_width' => '1',
            'clp_input-border_color' => '#7e8993',
            'clp_input-border_color_focus' => '#007cba',
            'clp_input-border_radius' => '3',
            'clp_inputs-showpassword' => '1',
            'clp_inputs-remember' => '1',
            'clp_button-background_color' => 'rgba(0,124,186,1)',
            'clp_button-background_color_hover' => 'rgba(0,124,186,1)',
            'clp_button-text_color' => '#ffffff',
            'clp_button-text_color_hover' => '#ffffff',
            'clp_button-font_size' => '14',
            'clp_button-width' => '30',
            'clp_button-height' => '40',
            'clp_button-align' => 'right',
            'clp_button-border_radius' => '3',
            'clp_button-border_width' => '1',
            'clp_button-border_color' => '#ffffff',
            'clp_button-border_color_hover' => '#ffffff',
            'clp_form_footer-align' => 'left',
            'clp_form_footer-font_size' => '13',
            'clp_form_footer-display_forgetpassword' => '1',
            'clp_form_footer-forgetpassword_color' => '#555d66',
            'clp_form_footer-forgetpassword_color_hover' => '#00a0d2',
            'clp_form_footer-display_backtoblog' => '1',
            'clp_form_footer-backtoblog_color' => '#555d66',
            'clp_form_footer-backtoblog_color_hover' => '#00a0d2',
            'clp_form_footer-register_color' => '#555d66',
            'clp_form_footer-register_color_hover' => '#00a0d2',
            'clp_form_footer-privacy_color' => '#555d66',
            'clp_form_footer-privacy_color_hover' => '#00a0d2',
        );
    }

    /**
     * Define CSS Customizer Settings fields
     * @since 1.0.0
     * @return array
    **/
    public function settings_fields() {

        $settings['layouts'] = array(
            'title'       => esc_html__( 'Layouts', 'clp-custom-login-page' ),
            'description' => '',
            'fields'      => array(
                array(
                    'id'            => 'clp_layouts-separator',
                    'label'         => esc_html__( 'Select Layout', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'          => 'clp_layouts',
                    'description' => '',
                    'type'        => 'clp-layout',
                    'default'     => 'default',
                    'choices'	=> array (
                        'default' => array(
                            'name'      => 'Default',
                            'thumbnail' => CLP_PLUGIN_PATH . 'assets/img/layouts/default.jpg',
                            'options' => $this->get_default_layout_settings()
                        ),
                        'light-theme' => array(
                            'name' => 'Light UI',
                            'thumbnail' => CLP_PLUGIN_PATH . 'assets/img/layouts/light-theme.jpg',
                            'options' => array(
                                'clp_logo' => 'none',
                                'clp_background-color' => '#ededed',
                                'clp_form-width' => '380',
                                'clp_form-height' => '480',
                                'clp_form-padding' => '30',
                                'clp_form-background' => 'rgba(255,255,255,1)',
                                'clp_form_typography-google_fonts' => '{"family":"Exo","variants":["100","200","300","regular","500","600","700","800","900","100italic","200italic","300italic","italic","500italic","600italic","700italic","800italic","900italic"],"selected":{"variant":"500"}}',
                                'clp_form-borders' => '1',
                                'clp_form-border_color' => '#828282',
                                'clp_form-border_radius' => '3',
                                'clp_form-shadow-spread_radius' => '-14',
                                'clp_form-shadow-color' => 'rgba(2,2,2,0.73)',
                                'clp_input-background_color_focus' => '#fff',
                                'clp_input-color' => '#232323',
                                'clp_input-color_focus' => '#232323',
                                'clp_input-label_color' => '#232323',
                                'clp_input-border_color' => '#a8a8a8',
                                'clp_input-border_color_focus' => '#4bb2f9',
                                'clp_button-background_color' => '#232323',
                                'clp_button-background_color_hover' => 'rgba(255,255,255,1)',
                                'clp_button-text_color_hover' => 'rgba(35,35,35,1)',
                                'clp_button-width' => '100',
                                'clp_button-border_color' => '#000000',
                                'clp_button-border_color_hover' => '#000000',
                                'clp_form_footer-forgetpassword_color' => '#b7b7b7',
                                'clp_form_footer-forgetpassword_color_hover' => '#4bb2f9',
                                'clp_form_footer-backtoblog_color' => '#b7b7b7',
                                'clp_form_footer-backtoblog_color_hover' => '#4bb2f9',
                                'clp_form_footer-register_color' => '#b7b7b7',
                                'clp_form_footer-register_color_hover' => '#4bb2f9',            
                                'clp_form_footer-privacy_color' => '#b7b7b7',
                                'clp_form_footer-privacy_color_hover' => '#4bb2f9',            
                            )
                        ),
                        'dark-theme' => array(
                            'name' => 'Dark UI',
                            'thumbnail' => CLP_PLUGIN_PATH . 'assets/img/layouts/dark-theme.jpg',
                            'options' => array(
                                'clp_logo' => 'none',
                                'clp_background-color' => '#212121',
                                'clp_form-width' => '380',
                                'clp_form-height' => '480',
                                'clp_form-padding' => '30',
                                'clp_form-background' => 'rgba(18,18,18,0.8)',
                                'clp_form_typography-google_fonts' => '{"family":"Exo","variants":["100","200","300","regular","500","600","700","800","900","100italic","200italic","300italic","italic","500italic","600italic","700italic","800italic","900italic"],"selected":{"variant":"500"}}',
                                'clp_form-borders' => '1',
                                'clp_form-border_color' => '#353535',
                                'clp_form-border_radius' => '3',
                                'clp_form-shadow-spread_radius' => '-14',
                                'clp_form-shadow-color' => 'rgba(2,2,2,0.73)',
                                'clp_input-background_color' => '#121212',
                                'clp_input-background_color_focus' => '#121212',
                                'clp_input-color' => '#ffffff',
                                'clp_input-color_focus' => '#ffffff',
                                'clp_input-label_color' => '#ffffff',
                                'clp_input-border_color' => '#272727',
                                'clp_input-border_color_focus' => '#4bb2f9',
                                'clp_button-background_color' => 'rgba(255,255,255,1)',
                                'clp_button-background_color_hover' => 'rgba(0,0,0,1)',
                                'clp_button-text_color' => '#000000',
                                'clp_button-width' => '100',
                                'clp_form_footer-forgetpassword_color' => '#b7b7b7',
                                'clp_form_footer-forgetpassword_color_hover' => '#4bb2f9',
                                'clp_form_footer-backtoblog_color' => '#b7b7b7',
                                'clp_form_footer-backtoblog_color_hover' => '#4bb2f9',
                                'clp_form_footer-register_color' => '#b7b7b7',
                                'clp_form_footer-register_color_hover' => '#4bb2f9',                                
                                'clp_form_footer-privacy_color' => '#b7b7b7',
                                'clp_form_footer-privacy_color_hover' => '#4bb2f9',                                
                            )
                        ),
                        'femine-theme' => array(
                            'name' => 'Feminine',
                            'thumbnail' => CLP_PLUGIN_PATH . 'assets/img/layouts/femine-theme.jpg',
                            'options' => array(
                                'clp_logo' => 'none',
                                'clp_background-color' => '#d10073',
                                'clp_form-width' => '380',
                                'clp_form-height' => '480',
                                'clp_form-padding' => '30',
                                'clp_form-background' => 'rgba(255,255,255,1)',
                                'clp_form-border_radius' => '3',
                                'clp_form_typography-google_fonts' => '{"family":"Exo","variants":["100","200","300","regular","500","600","700","800","900","100italic","200italic","300italic","italic","500italic","600italic","700italic","800italic","900italic"],"selected":{"variant":"500"}}',
                                'clp_form-borders' => '1',
                                'clp_form-border_color' => '#828282',
                                'clp_form-shadow-spread_radius' => '-14',
                                'clp_form-shadow-color' => 'rgba(2,2,2,0.73)',
                                'clp_input-background_color_focus' => '#fff',
                                'clp_input-color' => '#232323',
                                'clp_input-color_focus' => '#232323',
                                'clp_input-label_color' => '#232323',
                                'clp_input-border_color' => '#f7a5ca',
                                'clp_input-border_color_focus' => '#f72585',
                                'clp_button-background_color' => 'rgba(247,37,133,1)',
                                'clp_button-background_color_hover' => 'rgba(209,0,115,1)',
                                'clp_button-width' => '100',
                                'clp_button-height' => '48',
                                'clp_button-align' => 'center',
                                'clp_button-border_width' => '0',
                                'clp_form_footer-forgetpassword_color' => '#b7b7b7',
                                'clp_form_footer-forgetpassword_color_hover' => '#f72585',
                                'clp_form_footer-backtoblog_color' => '#b7b7b7',
                                'clp_form_footer-backtoblog_color_hover' => '#f72585',
                                'clp_form_footer-register_color' => '#b7b7b7',
                                'clp_form_footer-register_color_hover' => '#f72585',
                                'clp_form_footer-privacy_color' => '#b7b7b7',
                                'clp_form_footer-privacy_color_hover' => '#f72585',
                                                              
                            )
                        ),
                        'flat_white-theme' => array(
                            'name' => 'Flat White',
                            'thumbnail' => CLP_PLUGIN_PATH . 'assets/img/layouts/flat_white-theme.jpg',
                            'options' => array(
                                'clp_logo' => 'none',
                                'clp_background-color' => '#baa898',
                                'clp_form-width' => '380',
                                'clp_form-height' => '480',
                                'clp_form-padding' => '30',
                                'clp_form-background' => 'rgba(238,224,203,0.8)',
                                'clp_form-border_radius' => '3',
                                'clp_form_typography-google_fonts' => '{"family":"Exo","variants":["100","200","300","regular","500","600","700","800","900","100italic","200italic","300italic","italic","500italic","600italic","700italic","800italic","900italic"],"selected":{"variant":"500"}}',
                                'clp_form-borders' => '1',
                                'clp_form-border_color' => '#baa898',
                                'clp_form-shadow' => false,
                                'clp_input-background_color_focus' => '#fff',
                                'clp_input-color' => '#baa898',
                                'clp_input-color_focus' => '#baa898',
                                'clp_input-label_color' => '#2a2d34',
                                'clp_input-border_color' => '#baa898',
                                'clp_input-border_color_focus' => '#ba9575',
                                'clp_button-background_color' => 'rgba(9,122,198,1)',
                                'clp_button-background_color_hover' => 'rgba(42,45,52,1)',
                                'clp_button-width' => '100',
                                'clp_button-height' => '48',
                                'clp_button-align' => 'center',
                                'clp_button-border_color' => '#000000',
                                'clp_button-border_color_hover' => '#000000',
                                'clp_form_footer-forgetpassword_color' => '#848586',
                                'clp_form_footer-forgetpassword_color_hover' => '#2a2d34',
                                'clp_form_footer-backtoblog_color' => '#848586',
                                'clp_form_footer-backtoblog_color_hover' => '#2a2d34',
                                'clp_form_footer-register_color' => '#848586',
                                'clp_form_footer-register_color_hover' => '#2a2d34',
                                'clp_form_footer-privacy_color' => '#848586',
                                'clp_form_footer-privacy_color_hover' => '#2a2d34',
                                                              
                            )
                        ),
                    ),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => array( $this, 'sanitize_select' ),
                ),
            )
        );

        $settings['logo'] = array(
            'title'       => esc_html__( 'Logo', 'clp-custom-login-page' ),
            'description' => '',
            'fields'      => array(
                array(
                    'id'            => 'clp_logo-separator',
                    'label'         => esc_html__( 'Logo Type', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'          => 'clp_logo',
                    'label'       => esc_html__( 'Choose Logo Type', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'radio',
                    'default'     => 'image',
                    'choices'	=> array (
                        'none' 	    => esc_html__( 'Disabled', 'clp-custom-login-page' ),
                        'text' 	    => esc_html__( 'Text Logo', 'clp-custom-login-page' ),
                        'image' 	=> esc_html__( 'Image Logo', 'clp-custom-login-page' ),
                    ),
                    'transport'		=> 'refresh',
                    'sanitize_callback' => array( $this, 'sanitize_select' ),
                ),
                array(
                    'id'            => 'clp_logo-settings-separator',
                    'label'         => esc_html__( 'Logo Settings', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                    'active_callback' => array( $this, 'is_logo_enabled'),
                ),
                array(
                    'id'            => 'clp_logo-text',
                    'label'         => esc_html__( 'Logo Text', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'text',
                    'default'       => get_bloginfo('name'),
                    'transport'		=> 'postMessage',
                    'active_callback' => array( $this, 'is_logo_text'),
                    'sanitize_callback' => 'sanitize_text_field',
                ),
                array(
                    'id'          => 'clp_logo-text-color',
                    'label'       => esc_html__( 'Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#0073aa',
                    'transport'		=> 'postMessage',
                    'active_callback' => array( $this, 'is_logo_text'),
                    'sanitize_callback' => 'sanitize_hex_color',
                    'css'			=> array(
                        'selector'	=> '.clp-login-logo a, .clp-login-logo a:hover, .clp-login-logo a:focus',
                        'property'	=> 'color',
                        'dependency'=>  array('clp_logo', 'text'),
                    )
                ),

                array(
                    'id'            => 'clp_logo-image',
                    'label'         => esc_html__( 'Upload Custom Logo', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'media',
                    'mime_type'     => 'image',
                    'default'       => '',
                    'transport'		=> 'refresh',
                    'active_callback' => array( $this, 'is_logo_image'),
                    'sanitize_callback' =>  'absint',
                ),
                array(
                    'id'          => 'clp_logo-image-width',
                    'label'       => esc_html__( 'Max Width', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '80',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 1000,
                        'step'   => 5,
                        'suffix' => 'px', 
                    ),
                    'active_callback' => array( $this, 'is_logo_image'),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'css'			=> array(
                        'selector'	=> '.clp-login-logo img',
                        'property'	=> 'width',
                        'suffix'	=> 'px',
                        'dependency'=>  array('clp_logo', 'image'),
                    )
                ),
                array(
                    'id'          => 'clp_logo-spacing-top',
                    'label'       => esc_html__( 'Top Spacing', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '0',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 200,
                        'step'   => 5,
                        'suffix' => 'px', 
                    ),
                    'active_callback' => array( $this, 'is_logo_enabled'),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'css'			=> array(
                        'selector'	=> '.login .clp-login-logo',
                        'property'	=> 'margin-top',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'          => 'clp_logo-spacing-bottom',
                    'label'       => esc_html__( 'Bottom Spacing', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '30',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 200,
                        'step'   => 5,
                        'suffix' => 'px', 
                    ),
                    'active_callback' => array( $this, 'is_logo_enabled'),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'css'			=> array(
                        'selector'	=> '.login .clp-login-logo',
                        'property'	=> 'margin-bottom',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'            => 'clp_logo-url',
                    'label'         => esc_html__( 'Change Logo Link', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'text',
                    'default'       => get_bloginfo('url'),
                    'transport'		=> 'postMessage',
                    'active_callback' => array( $this, 'is_logo_enabled'),
                    'sanitize_callback' => 'esc_url',
                ),
                array(
                    'id'            => 'clp_logo-contained',
                    'label'         => esc_html__( 'Logo inside Form Container?', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'toggle',
                    'transport'		=> 'postMessage',
                    'default'       => false,
                    'sanitize_callback' => array( $this, 'sanitize_toggle' ),
                    'active_callback' => array( $this, 'is_logo_enabled'),
                ),
                array(
                    'id'            => 'clp_logo-typography-separator',
                    'label'         => esc_html__( 'Logo Typography', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                    'active_callback' => array( $this, 'is_logo_text'),
                ),
                array(
                    'id'          => 'clp_logo-google_fonts',
                    'description' => '',
                    'type'        => 'google-fonts',
                    'transport'	    => 'postMessage',
                    'default'       => '{"family":"Roboto","variants":["100","100italic","300","300italic","regular","italic","500","500italic","700","700italic","900","900italic"],"selected":{"variant":"regular"}}',
                    'active_callback' => array( $this, 'is_logo_text'),
                    'sanitize_callback' => 'sanitize_text_field',
                ),
                array(
                    'id'          => 'clp_logo-text-font_size',
                    'label'       => esc_html__( 'Logo Font Size', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '35',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 10,
                        'max'    => 150,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'active_callback' => array( $this, 'is_logo_text'),
                    'sanitize_callback' => 'absint',
                    'css'			=> array(
                        'selector'	=> '.login .clp-login-logo',
                        'property'	=> 'font-size',
                        'suffix'	=> 'px',
                        'dependency'=>  array('clp_logo', 'text'),
                    )
                ),
                array(
                    'id'          => 'clp_logo-text-letter_spacing',
                    'label'       => esc_html__( 'Letter Spacing', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '0',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => -50,
                        'max'    => 50,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'active_callback' => array( $this, 'is_logo_text'),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'css'			=> array(
                        'selector'	=> '.clp-login-logo',
                        'property'	=> 'letter-spacing',
                        'suffix'    => 'px'
                    )
                ),
            ),
        );

        $settings['background'] = array(
            'title'       => esc_html__( 'Background', 'clp-custom-login-page' ),
            'description' => '',
            'fields'      => array(
                array(
                    'id'            => 'clp_background-separator',
                    'label'         => esc_html__( 'Background', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'          => 'clp_background',
                    'label'       => esc_html__( 'Select Background', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'radio',
                    'default'     => 'color',
                    'choices'	=> array (
                        'color' 	=> esc_html__( 'Color', 'clp-custom-login-page' ),
                        'gradient' 	=> esc_html__( 'Gradient', 'clp-custom-login-page' ),
                        'pattern' 	=> esc_html__( 'Pattern', 'clp-custom-login-page' ),
                        'image' 	=> esc_html__( 'Image', 'clp-custom-login-page' ),
                        'video' 	=> esc_html__( 'Video', 'clp-custom-login-page' ),
                    ),
                    'transport'		=> 'refresh',
                    'sanitize_callback' => array( $this, 'sanitize_select' ),
                ),
                array(
                    'id'            => 'clp_background-settings-separator',
                    'label'         => esc_html__( 'Background Settings', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'          => 'clp_background-color',
                    'label'       => esc_html__( 'Background Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#f1f1f1',
                    'transport'		=> 'postMessage',
                    'active_callback' =>  array( $this, 'is_background_color'),
                    'sanitize_callback' => 'sanitize_hex_color',
                    'css'			=> array(
                        'selector'	=> '.login-background',
                        'property'	=> 'background-color',
                        'dependency'=>  array('clp_background', 'color')
                    )
                ),
                array(
                    'id'          => 'clp_background-gradient-color1',
                    'label'       => esc_html__( 'Gradient Start Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#f1f1f1',
                    'transport'		=> 'postMessage',
                    'active_callback' =>  array( $this, 'is_background_gradient'),
                    'sanitize_callback' => 'sanitize_hex_color',
                    'css'			=> array(
                        'selector'	=> '.login-background',
                        'property'	=> 'background',
                        'dependency'=>  array('clp_background', 'gradient'),
                        'css_value'     => sprintf('linear-gradient(%sdeg, %s %s%%, %%VALUE%% %s%%)', get_option('clp_background-gradient-angle', '30'), get_option('clp_background-gradient-color2', '#f1f1f1'), get_option('clp_background-gradient-color1-position', '0'), get_option('clp_background-gradient-color2-position', '100'))
                    )
                ),
                array(
                    'id'          => 'clp_background-gradient-color1-position',
                    'label'       => esc_html__( 'Gradient Start Position', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '0',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 100,
                        'step'   => 1,
                        'suffix' => '%', 
                    ),
                    'active_callback' =>  array( $this, 'is_background_gradient'),
                    'sanitize_callback' => 'absint',
                ),
                array(
                    'id'          => 'clp_background-gradient-color2',
                    'label'       => esc_html__( 'Gradient End Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#878787',
                    'transport'		=> 'postMessage',
                    'active_callback' =>  array( $this, 'is_background_gradient'),
                    'sanitize_callback' => 'sanitize_hex_color',
                ),
                array(
                    'id'          => 'clp_background-gradient-color2-position',
                    'label'       => esc_html__( 'Gradient End Position', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '100',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 100,
                        'step'   => 1,
                        'suffix' => '%', 
                    ),
                    'active_callback' =>  array( $this, 'is_background_gradient'),
                    'sanitize_callback' => 'absint',
                ),
                array(
                    'id'          => 'clp_background-gradient-angle',
                    'label'       => esc_html__( 'Gradient Angle', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '30',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 1,
                        'max'    => 360,
                        'step'   => 1,
                        'suffix' => 'deg', 
                    ),
                    'active_callback' =>  array( $this, 'is_background_gradient'),
                    'sanitize_callback' => 'absint',
                ),
                array(
                    'id'          => 'clp_background-pattern',
                    'label'       => esc_html__( 'Select Pattern', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'select',
                    'default'     => 'fabric',
                    'choices'	=> array (
                        'fabric' 				=> esc_html__( 'Fabric', 'clp-custom-login-page' ),
                        'gray_sand' 			=> esc_html__( 'Gray Sand', 'clp-custom-login-page' ),
                        'green_dust_scratch' 	=> esc_html__( 'Green Dust Scratch', 'clp-custom-login-page' ),
                        'mirrored_squares' 		=> esc_html__( 'Mirrored Squares', 'clp-custom-login-page' ),
                        'noisy' 				=> esc_html__( 'Noisy', 'clp-custom-login-page' ),
                        'photography' 			=> esc_html__( 'Photography', 'clp-custom-login-page' ),
                        'playstation' 			=> esc_html__( 'Playstation', 'clp-custom-login-page' ),
                        'sakura' 				=> esc_html__( 'Sakura', 'clp-custom-login-page' ),
                        'white_sand' 			=> esc_html__( 'White Sand', 'clp-custom-login-page' ),
                        'white_texture' 		=> esc_html__( 'White Texture', 'clp-custom-login-page' ),
                        'custom' 		        => esc_html__( 'Custom', 'clp-custom-login-page' ),
                    ),
                    'transport'		=> 'postMessage',
                    'active_callback' =>  array( $this, 'is_background_pattern'),
                    'sanitize_callback' => array( $this, 'sanitize_select' ),
                    'css'			=> array(
                        'selector'	=> '.login-background',
                        'property'	=> 'background-image',
                        'dependency'=>  array('clp_background', 'pattern'),
                        'css_value'     => 'url("'.CLP_PLUGIN_PATH.'assets/img/patterns/%VALUE%.png")',
                    )
                ),
                array(
                    'id'          => 'clp_background-pattern-custom',
                    'label'       => esc_html__( 'Upload Custom Pattern', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'media',
                    'mime_type'    => 'image',
                    'default'     => '',
                    'transport'		=> 'postMessage',
                    'active_callback' =>  array( $this, 'is_custom_pattern'),
                    'sanitize_callback' =>  'absint',
                    'css'			=> array(
                        'selector'	=> '.clp-pattern-custom .login-background',
                        'property'	=> 'background-image',
                        'dependency'=>  array('clp_background', 'pattern'),
                        'css_value'     => 'url("%VALUE%")',
                        'get_value' => array('wp_get_attachment_url', get_option( 'clp_background-pattern-custom', '' ) ),
        
                    )
                ),
                array(
                    'id'          => 'clp_background-unsplash',
                    'label'       => esc_html__( 'Unsplash Gallery', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'unsplash',
                    'default'     => '',
                    'transport'		=> 'postMessage',
                    'active_callback' =>  array( $this, 'is_background_image'),
                    'sanitize_callback' =>  'sanitize_text_field',
                    'css'			=> array(
                        'selector'	=> '.login-background',
                        'property'	=> 'background-image',
                        'dependency'=>  array('clp_background', 'image'),
                        'css_value'     => 'url("%VALUE%")',
                        'get_value' => array('CLP_Unsplash_Api::get_unsplash_image', get_option( 'clp_background-unsplash', '' ) ),
                    )
                ),
                array(
                    'id'          => 'clp_background-image',
                    'label'       => esc_html__( 'Upload Custom Image', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'media',
                    'mime_type'    => 'image',
                    'default'     => '',
                    'transport'		=> 'postMessage',
                    'active_callback' =>  array( $this, 'is_background_image'),
                    'sanitize_callback' =>  'absint',
                    'css'			=> array(
                        'selector'	=> '.login-background',
                        'property'	=> 'background-image',
                        'dependency'=>  array('clp_background', 'image'),
                        'css_value'     => 'url("%VALUE%")',
                        'get_value' => array('wp_get_attachment_image_src', get_option( 'clp_background-image', '' ), CLP_Helper_Functions::get_image_size(), 1),
                        'return_value' => array('array', 0)
                    )
                ),
                array(
                    'id'          => 'clp_background-video_local',
                    'label'       => esc_html__( 'Upload Custom Video', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'media',
                    'mime_type'    => 'video',
                    'default'     => '',
                    'transport'		=> 'refresh',
                    'active_callback' =>  array( $this, 'is_background_video'),
                    'sanitize_callback' =>  'absint',
                ),
                array(
                    'id'          => 'clp_background-video_yt',
                    'label'       => esc_html__( 'Insert YouTube URL', 'clp-custom-login-page' ),
                    'description' => esc_html__( 'YouTube background video is not supported on mobile devices. Image placeholder will be used instead.', 'clp-custom-login-page' ),
                    'type'        => 'text',
                    'default'     => '',
                    'transport'		=> 'refresh',
                    'active_callback' =>  array( $this, 'is_background_video'),
                    'sanitize_callback' =>  'esc_url',
                ),
                array(
                    'id'          => 'clp_background-video_loop',
                    'label'       => esc_html__( 'Loop video', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'toggle',
                    'default'     => true,
                    'transport'		=> 'refresh',
                    'active_callback' =>  array( $this, 'is_background_video'),
                    'sanitize_callback' =>  'absint',
                ),
                array(
                    'id'          => 'clp_background-video_thumb',
                    'label'       => esc_html__( 'Upload Video Thumbnail', 'clp-custom-login-page' ),
                    'description' => esc_html__( 'This image will be used as image placeholder before the video loads and also displayed on mobile devices.', 'clp-custom-login-page' ),
                    'type'        => 'media',
                    'mime_type'    => 'image',
                    'default'     => '',
                    'transport'		=> 'postMessage',
                    'active_callback' =>  array( $this, 'is_background_video'),
                    'sanitize_callback' =>  'absint',
                ),
                array(
                    'id'            => 'clp_background-overlay-separator',
                    'label'         => esc_html__( 'Background Overlay', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'            => 'clp_background-overlay-enable',
                    'label'         => esc_html__( 'Enable Overlay', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'toggle',
                    'transport'		=> 'postMessage',
                    'default'       => false,
                    'sanitize_callback' => array( $this, 'sanitize_toggle' ),
                ),
                array(
                    'id'          => 'clp_background-overlay-color',
                    'label'       => esc_html__( 'Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'alpha-color',
                    'default'     => 'rgba(0,0,0,0.4)',
                    'transport'		=> 'postMessage',
                    'active_callback' => array( $this, 'is_overlay_enabled' ),
                    'sanitize_callback' => array ( $this, 'sanitize_alpha_color'),
                    'css'			=> array(
                        'selector'	=> '.login-overlay',
                        'property'	=> 'background-color',
                        'dependency'=>  array('clp_background-overlay-enable', '1'),
                    )
                ),

                array(
                    'id'            => 'clp_background-effects-separator',
                    'label'         => esc_html__( 'Background Effects', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'          => 'clp_background-blur',
                    'label'       => esc_html__( 'Blur', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '0',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 10,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int' ),
                    'css'			=> array(
                        'selector'	=> '.login-background',
                        'property'	=> 'filter',
                        'css_value'     => 'blur(%VALUE%px)',
                    )
                ),
            )
        );

        $settings['form'] = array(
            'title'       => esc_html__( 'Form', 'clp-custom-login-page' ),
            'description' => '',
            'fields'      => array(
                array(
                    'id'            => 'clp_form-separator',
                    'label'         => esc_html__( 'Customize Form', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'          => 'clp_form-width',
                    'label'       => esc_html__( 'Width', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '340',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 300,
                        'max'    => 1000,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => 'absint',
                    'css'			=> array(
                        'selector'	=> '.login #login',
                        'property'	=> 'width',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'          => 'clp_form-height',
                    'label'       => esc_html__( 'Minimum Height', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '380',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 300,
                        'max'    => 1000,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => 'absint',
                    'css'			=> array(
                        'selector'	=> '.login #login',
                        'property'	=> 'min-height',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'          => 'clp_form-padding',
                    'label'       => esc_html__( 'Form Spacing', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '20',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 150,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => 'absint',
                    'css'			=> array(
                        'selector'	=> '.login #login',
                        'property'	=> 'padding',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'          => 'clp_form-background',
                    'label'       => esc_html__( 'Form Background Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'alpha-color',
                    'default'     => 'rgba(255,255,255,1)',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => array ( $this, 'sanitize_alpha_color'),
                    'css'			=> array(
                        'selector'	=> '.login #login',
                        'property'	=> 'background-color',
                    )
                ),
                array(
                    'id'          => 'clp_form-blur',
                    'label'       => esc_html__( 'Backdrop Blur', 'clp-custom-login-page' ),
                    'description' => 'Too see this effect, make sure Form Background Color is opaque.',
                    'type'        => 'range',
                    'default'     => '0',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 40,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'css'			=> array(
                        'selector'	=> '.login #login',
                        'property'	=> '-webkit-backdrop-filter, backdrop-filter',
                        'css_value'    => 'blur(%VALUE%px)'
                    )
                ),
                array(
                    'id'          => 'clp_form-border_radius',
                    'label'       => esc_html__( 'Border Radius', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '0',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 100,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'css'			=> array(
                        'selector'	=> '.login #login',
                        'property'	=> 'border-radius',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'            => 'clp_form_typography-separator',
                    'label'         => esc_html__( 'Typography', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'          => 'clp_form_typography-google_fonts',
                    'description' => '',
                    'type'        => 'google-fonts',
                    'transport'	    => 'postMessage',
                    'default'       => '{"family":"Roboto","variants":["100","100italic","300","300italic","regular","italic","500","500italic","700","700italic","900","900italic"],"selected":{"variant":"regular"}}',
                    'sanitize_callback' => 'sanitize_text_field',
                ),

                array(
                    'id'            => 'clp_form_borders-separator',
                    'label'         => esc_html__( 'Form Borders', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'            => 'clp_form-borders',
                    'label'         => esc_html__( 'Enable Borders', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'toggle',
                    'transport'		=> 'postMessage',
                    'default'       => false,
                    'sanitize_callback' => array( $this, 'sanitize_toggle' ),
                ),
                array(
                    'id'          => 'clp_form-border_color',
                    'label'       => esc_html__( 'Border Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#7e8993',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'active_callback'=>  array( $this, 'is_form_borders' ),
                ),
                array(
                    'id'          => 'clp_form-border_width',
                    'label'       => esc_html__( 'Border Width', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '1',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 1,
                        'max'    => 10,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => 'absint',
                    'active_callback'=>  array( $this, 'is_form_borders' ),
                    'css'			=> array(
                        'selector'	=> '.login #login',
                        'property'	=> 'border',
                        'css_value'    => '%VALUE%px solid '.get_option('clp_form-border_color', '#7e8993'),
                        'dependency'=>  array('clp_form-borders', '1'),
                    )
                ),
                array(
                    'id'            => 'clp_form_shadow-separator',
                    'label'         => esc_html__( 'Form Shadow', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'            => 'clp_form-shadow',
                    'label'         => esc_html__( 'Enable Shadow', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'toggle',
                    'transport'		=> 'postMessage',
                    'default'       => '1',
                    'sanitize_callback' => array( $this, 'sanitize_toggle' ),
                ),
                array(
                    'id'          => 'clp_form-shadow-horizontal_length',
                    'label'       => esc_html__( 'Horizontal Length', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '10',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => -200,
                        'max'    => 200,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'active_callback'=>  array( $this, 'is_form_shadow' ),
                ),
                array(
                    'id'          => 'clp_form-shadow-vertical_length',
                    'label'       => esc_html__( 'Vertical Length', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '10',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => -200,
                        'max'    => 200,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    // 'sanitize_callback' => 'absint',
                    'active_callback'=>  array( $this, 'is_form_shadow' ),
                ),
                array(
                    'id'          => 'clp_form-shadow-blur_radius',
                    'label'       => esc_html__( 'Blur Radius', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '40',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 200,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'active_callback'=>  array( $this, 'is_form_shadow' ),
                ),
                array(
                    'id'          => 'clp_form-shadow-spread_radius',
                    'label'       => esc_html__( 'Spread Radius', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '-10',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => -200,
                        'max'    => 200,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'active_callback'=>  array( $this, 'is_form_shadow' ),
                ),
                array(
                    'id'          => 'clp_form-shadow-color',
                    'label'       => esc_html__( 'Shadow Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'alpha-color',
                    'default'     => 'rgba(0,0,0,0.75)',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => array( $this, 'sanitize_alpha_color' ),
                    'active_callback'=>  array( $this, 'is_form_shadow' ),
                    'css'			=> array(
                        'selector'	=> '.login #login',
                        'property'	=> 'box-shadow',
                        'css_value'    => get_option('clp_form-shadow-horizontal_length', '10').'px '.get_option('clp_form-shadow-vertical_length', '10').'px '.get_option('clp_form-shadow-blur_radius', '40').'px '.get_option('clp_form-shadow-spread_radius', '-10').'px %VALUE%',
                        'dependency'=>  array('clp_form-shadow', '1'),
                    )
                ),
            ),
        );

        $settings['inputs'] = array(
            'title'       => esc_html__( 'Inputs', 'clp-custom-login-page' ),
            'description' => '',
            'fields'      => array(
                array(
                    'id'            => 'clp_inputs-separator',
                    'label'         => esc_html__( 'Customize Inputs and Labels', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'          => 'clp_input-background_color',
                    'label'       => esc_html__( 'Background Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'alpha-color',
                    'default'     => '#ffffff',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => array( $this, 'sanitize_alpha_color' ),
                    'css'			=> array(
                        'selector'	=> '.login .clp-login-form-container form .input, .login input[type="text"], .login .clp-login-form-container form input[type="checkbox"]',
                        'property'	=> 'background-color',
                    )
                ),
                array(
                    'id'          => 'clp_input-background_color_focus',
                    'label'       => esc_html__( 'Focus Background Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'alpha-color',
                    'default'     => '#ffffff',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => array( $this, 'sanitize_alpha_color' ),
                    'css'			=> array(
                        'selector'	=> '.login input[type="text"]:focus, .login input[type="password"]:focus',
                        'property'	=> 'background-color',
                    )
                ),
                array(
                    'id'          => 'clp_input-color',
                    'label'       => esc_html__( 'Text Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#32373c',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'css'			=> array(
                        'selector'	=> '.login input[type="text"], .login input[type="password"], .login .button.wp-hide-pw .dashicons',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'          => 'clp_input-color_focus',
                    'label'       => esc_html__( 'Focus Text Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#32373c',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'css'			=> array(
                        'selector'	=> '.login input[type="text"]:focus, .login input[type="password"]:focus, .login input[type="checkbox"]:focus',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'          => 'clp_input-label_color',
                    'label'       => esc_html__( 'Labels Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#32373c',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'css'			=> array(
                        'selector'	=> '.login label',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'          => 'clp_input-font_size_label',
                    'label'       => esc_html__( 'Label Font Size', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '14',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 10,
                        'max'    => 50,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => 'sanitize_text_field',
                    'css'			=> array(
                        'selector'	=> '.login label',
                        'property'	=> 'font-size',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'          => 'clp_input-font_size_input',
                    'label'       => esc_html__( 'Input Font Size', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '24',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 10,
                        'max'    => 50,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => 'sanitize_text_field',
                    'css'			=> array(
                        'selector'	=> '.login form .input, .login input[type="text"], .login input[type="password"]',
                        'property'	=> 'font-size',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'          => 'clp_input-height',
                    'label'       => esc_html__( 'Minimum Height', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '40',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 1,
                        'max'    => 100,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => 'sanitize_text_field',
                    'css'			=> array(
                        'selector'	=> '.login input[type="text"], .login input[type="password"], .login .clp-form-container .button.wp-hide-pw',
                        'property'	=> 'min-height',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'          => 'clp_input-border_width',
                    'label'       => esc_html__( 'Border Width', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '1',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 10,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'css'			=> array(
                        'selector'	=> '.login .clp-form-container input[type="text"], .login .clp-form-container input[type="password"], .login .clp-form-container input[type="checkbox"]',
                        'property'	=> 'border',
                        'css_value'    => '%VALUE%px solid '.get_option('clp_input-border_color', '#7e8993'),
                    )
                ),
                array(
                    'id'          => 'clp_input-border_color',
                    'label'       => esc_html__( 'Border Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#7e8993',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                ),
                array(
                    'id'          => 'clp_input-border_color_focus',
                    'label'       => esc_html__( 'Focus Border Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#007cba',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'css'			=> array(
                        'selector'	=> '.login .clp-form-container input[type="text"]:focus, .login .clp-form-container input[type="password"]:focus, .login input[type="checkbox"]:focus',
                        'property'	=> 'border-color',
                    )
                ),
                array(
                    'id'          => 'clp_input-border_radius',
                    'label'       => esc_html__( 'Border Radius', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '3',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 50,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'css'			=> array(
                        'selector'	=> '.login input[type="text"], .login input[type="password"], .login input[type="checkbox"]',
                        'property'	=> 'border-radius',
                        'suffix'	=> 'px',
                    )
                ),
                array(
                    'id'            => 'clp_inputs-showpassword',
                    'label'         => esc_html__( 'Display "Show Password" Icon?', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'toggle',
                    'transport'		=> 'postMessage',
                    'default'       => '1',
                    'sanitize_callback' => array( $this, 'sanitize_toggle' ),
                ),
                array(
                    'id'            => 'clp_inputs-remember-separator',
                    'label'         => esc_html__( 'Remember Me', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'            => 'clp_inputs-remember',
                    'label'         => esc_html__( 'Display "Remember Me" Checkbox?', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'toggle',
                    'transport'		=> 'postMessage',
                    'default'       => '1',
                    'sanitize_callback' => array( $this, 'sanitize_toggle' ),
                ),
                array(
                    'id'            => 'clp_inputs-remember_text',
                    'label'         => esc_html__( 'Custom "Remember Me" Text', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'text',
                    'default'       => __('Remember Me', 'clp-custom-login-page'),
                    'transport'		=> 'postMessage',
                    'active_callback' => array( $this, 'is_rememberme'),
                    'sanitize_callback' => 'sanitize_text_field',
                ),
            ),
        );

        $settings['button'] = array(
            'title'       => esc_html__( 'Button', 'clp-custom-login-page' ),
            'description' => '',
            'fields'      => array(
                array(
                    'id'            => 'clp_button-separator',
                    'label'         => esc_html__( 'Customize Buttons', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'            => 'clp_button-text',
                    'label'         => esc_html__( 'Button Text', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'text',
                    'default'       => __('Log In', 'clp-custom-login-page'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_text_field',
                ),
                array(
                    'id'          => 'clp_button-background_color',
                    'label'       => esc_html__( 'Background Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'alpha-color',
                    'default'     => 'rgba(0,124,186,1)',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => array($this, 'sanitize_alpha_color'),
                    'css'			=> array(
                        'selector'	=> '.wp-core-ui .clp-form-container .button-primary',
                        'property'	=> 'background-color',
                    )
                ),
                array(
                    'id'          => 'clp_button-background_color_hover',
                    'label'       => esc_html__( 'Background Hover Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'alpha-color',
                    'default'     => 'rgba(0,124,186,1)',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => array($this, 'sanitize_alpha_color'),
                    'css'			=> array(
                        'selector'	=> '.wp-core-ui .clp-form-container .button-primary:hover, .wp-core-ui .clp-form-container .button-primary:active, .wp-core-ui .clp-form-container .button-primary:focus',
                        'property'	=> 'background-color',
                    )
                ),
                array(
                    'id'          => 'clp_button-text_color',
                    'label'       => esc_html__( 'Text Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#ffffff',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'css'			=> array(
                        'selector'	=> '.wp-core-ui .clp-form-container .button-primary',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'          => 'clp_button-text_color_hover',
                    'label'       => esc_html__( 'Text Hover Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#ffffff',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'css'			=> array(
                        'selector'	=> '.wp-core-ui .clp-form-container .button-primary:hover, .wp-core-ui .clp-form-container .button-primary:active, .wp-core-ui .clp-form-container .button-primary:focus',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'          => 'clp_button-font_size',
                    'label'       => esc_html__( 'Font Size', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '14',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 10,
                        'max'    => 50,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => 'absint',
                    'css'			=> array(
                        'selector'	=> '.wp-core-ui .clp-form-container .button-primary',
                        'property'	=> 'font-size',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'          => 'clp_button-width',
                    'label'       => esc_html__( 'Width', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '30',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 1,
                        'max'    => 100,
                        'step'   => 1,
                        'suffix' => '%', 
                    ),
                    'sanitize_callback' => 'absint',
                    'css'			=> array(
                        'selector'	=> '.wp-core-ui .clp-form-container .button-primary',
                        'property'	=> 'min-width',
                        'suffix'    => '%'
                    )
                ),
                array(
                    'id'          => 'clp_button-height',
                    'label'       => esc_html__( 'Minimum Height', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '40',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 1,
                        'max'    => 100,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => 'absint',
                    'css'			=> array(
                        'selector'	=> '.wp-core-ui .clp-form-container .button-primary.button-large',
                        'property'	=> 'min-height',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'          => 'clp_button-align',
                    'label'       => esc_html__( 'Alignment', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'select',
                    'default'     => 'right',
                    'choices'	=> array (
                        'left' 		=> esc_html__( 'Left', 'clp-custom-login-page' ),
                        'center' 	=> esc_html__( 'Center', 'clp-custom-login-page' ),
                        'right' 	=> esc_html__( 'Right', 'clp-custom-login-page' ),
                    ),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => array( $this, 'sanitize_select' ),
                    'css'			=> array(
                        'selector'	=> '#login form p.submit',
                        'property'	=> 'text-align',
                    )
                ),
                array(
                    'id'          => 'clp_button-border_radius',
                    'label'       => esc_html__( 'Border Radius', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '3',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 50,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'css'			=> array(
                        'selector'	=> '.wp-core-ui .clp-form-container .button-primary',
                        'property'	=> 'border-radius',
                        'suffix'	=> 'px',
                    )
                ),
                array(
                    'id'          => 'clp_button-border_width',
                    'label'       => esc_html__( 'Border Width', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '1',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 0,
                        'max'    => 10,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => array( $this, 'sanitize_negative_int'),
                    'css'			=> array(
                        'selector'	=> '.wp-core-ui .clp-form-container .button-primary',
                        'property'	=> 'border',
                        'css_value'    => '%VALUE%px solid '.get_option('clp_button-border_color', '#fff'),
                    )
                ),
                array(
                    'id'          => 'clp_button-border_color',
                    'label'       => esc_html__( 'Border Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#ffffff',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                ),
                array(
                    'id'          => 'clp_button-border_color_hover',
                    'label'       => esc_html__( 'Border Hover Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#ffffff',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'css'			=> array(
                        'selector'	=> '.wp-core-ui .clp-form-container .button-primary:hover, .wp-core-ui .clp-form-container .button-primary:active, .wp-core-ui .clp-form-container .button-primary:focus',
                        'property'	=> 'border-color',
                    )
                ),
            ),
        );

        $settings['form_footer'] = array(
            'title'       => esc_html__( 'Form Footer', 'clp-custom-login-page' ),
            'description' => '',
            'fields'      => array(
                array(
                    'id'            => 'clp_form_footer-separator',
                    'label'         => esc_html__( 'Customize Form Footer', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'          => 'clp_form_footer-align',
                    'label'       => esc_html__( 'Text Align', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'select',
                    'default'     => 'left',
                    'choices'	=> array (
                        'left' 		=> esc_html__( 'Left', 'clp-custom-login-page' ),
                        'center' 	=> esc_html__( 'Center', 'clp-custom-login-page' ),
                        'right' 	=> esc_html__( 'Right', 'clp-custom-login-page' ),
                    ),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => array( $this, 'sanitize_select' ),
                ),
                array(
                    'id'          => 'clp_form_footer-font_size',
                    'label'       => esc_html__( 'Font Size', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'range',
                    'default'     => '13',
                    'transport'		=> 'postMessage',
                    'input_attrs' => array(
                        'min'    => 10,
                        'max'    => 50,
                        'step'   => 1,
                        'suffix' => 'px', 
                    ),
                    'sanitize_callback' => 'sanitize_text_field',
                    'css'			=> array(
                        'selector'	=> '.clp-login.login #nav, .clp-login.login #backtoblog, .privacy-policy-link',
                        'property'	=> 'font-size',
                        'suffix'    => 'px'
                    )
                ),
                array(
                    'id'            => 'clp_form_footer-forget_password-separator',
                    'label'         => esc_html__( 'Forget Password', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'            => 'clp_form_footer-display_forgetpassword',
                    'label'         => esc_html__( 'Display "Forget Password"?', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'toggle',
                    'transport'		=> 'postMessage',
                    'default'       => '1',
                    'sanitize_callback' => array( $this, 'sanitize_toggle' ),
                ),
                array(
                    'id'          => 'clp_form_footer-forgetpassword_color',
                    'label'       => esc_html__( 'Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#555d66',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'active_callback' => array( $this, 'is_forgetpassword'),
                    'css'			=> array(
                        'selector'	=> '.login #nav a:last-of-type',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'          => 'clp_form_footer-forgetpassword_color_hover',
                    'label'       => esc_html__( 'Hover Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#00a0d2',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'active_callback' => array( $this, 'is_forgetpassword'),
                    'css'			=> array(
                        'selector'	=> '.login #nav a:last-of-type:hover',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'            => 'clp_form_footer-forgetpassword_text',
                    'label'         => esc_html__( 'Custom "Forget Password" Text', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'text',
                    'default'       => __( 'Lost your password?', 'clp-custom-login-page' ),
                    'transport'		=> 'postMessage',
                    'active_callback' => array( $this, 'is_forgetpassword'),
                    'sanitize_callback' => 'sanitize_text_field',
                ),
                
                array(
                    'id'            => 'clp_form_footer-login_text',
                    'label'         => esc_html__( 'Custom "Log In" Text', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'text',
                    'default'       => __( 'Log In', 'clp-custom-login-page' ),
                    'transport'		=> 'postMessage',
                    'active_callback' => array( $this, 'is_forgetpassword'),
                    'sanitize_callback' => 'sanitize_text_field',
                ),
                array(
                    'id'            => 'clp_form_footer-backtoblog-separator',
                    'label'         => esc_html__( 'Back to Site', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'            => 'clp_form_footer-display_backtoblog',
                    'label'         => esc_html__( 'Display "Back to Site"?', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'toggle',
                    'transport'		=> 'postMessage',
                    'default'       => '1',
                    'sanitize_callback' => array( $this, 'sanitize_toggle' ),
                ),
                array(
                    'id'          => 'clp_form_footer-backtoblog_color',
                    'label'       => esc_html__( 'Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#555d66',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'active_callback' => array( $this, 'is_backtoblog'),
                    'css'			=> array(
                        'selector'	=> '#login #backtoblog a',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'          => 'clp_form_footer-backtoblog_color_hover',
                    'label'       => esc_html__( 'Hover Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#00a0d2',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'active_callback' => array( $this, 'is_backtoblog'),
                    'css'			=> array(
                        'selector'	=> '#login #backtoblog a:hover',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'            => 'clp_form_footer-backtoblog_text',
                    'label'         => esc_html__( 'Custom "Back to Site" Text', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'text',
                    'default'       => sprintf( _x( '&larr; Back to %s', 'site' ), get_bloginfo( 'title', 'display' ) ),
                    'transport'		=> 'postMessage',
                    'active_callback' => array( $this, 'is_backtoblog'),
                    'sanitize_callback' => 'sanitize_text_field',
                ),
                array(
                    'id'            => 'clp_form_footer-privacy-separator',
                    'label'         => esc_html__( 'Privacy Link', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                    'active_callback' => array( $this, 'is_privacy_link'),
                ),
                array(
                    'id'          => 'clp_form_footer-privacy_color',
                    'label'       => esc_html__( 'Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#555d66',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'active_callback' => array( $this, 'is_privacy_link'),
                    'css'			=> array(
                        'selector'	=> '.privacy-policy-link',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'          => 'clp_form_footer-privacy_color_hover',
                    'label'       => esc_html__( 'Hover Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#00a0d2',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'active_callback' => array( $this, 'is_privacy_link'),
                    'css'			=> array(
                        'selector'	=> '.privacy-policy-link:hover',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'            => 'clp_form_footer-register-separator',
                    'label'         => esc_html__( 'Register', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                    'active_callback' => array( $this, 'can_register'),
                ),
                array(
                    'id'            => 'clp_form_footer-display_register',
                    'label'         => esc_html__( 'Display "Register"?', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'toggle',
                    'transport'		=> 'postMessage',
                    'default'       => '1',
                    'sanitize_callback' => array( $this, 'sanitize_toggle' ),
                ),
                array(
                    'id'          => 'clp_form_footer-register_color',
                    'label'       => esc_html__( 'Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#555d66',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'active_callback' => array( $this, 'is_register'),
                    'css'			=> array(
                        'selector'	=> '.login #nav a:first-of-type, .login #nav',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'          => 'clp_form_footer-register_color_hover',
                    'label'       => esc_html__( 'Hover Color', 'clp-custom-login-page' ),
                    'description' => '',
                    'type'        => 'color',
                    'default'     => '#00a0d2',
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color',
                    'active_callback' => array( $this, 'is_register'),
                    'css'			=> array(
                        'selector'	=> '.login #nav a:first-of-type:hover',
                        'property'	=> 'color',
                    )
                ),
                array(
                    'id'            => 'clp_form_footer-register_text',
                    'label'         => esc_html__( 'Custom "Register" Text', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'text',
                    'default'       => __( 'Register', 'clp-custom-login-page' ),
                    'transport'		=> 'postMessage',
                    'active_callback' => array( $this, 'is_register'),
                    'sanitize_callback' => 'sanitize_text_field',
                ),
                array(
                    'id'            => 'clp_form_footer-login_link_separator',
                    'label'         => esc_html__( 'Login Link Separator', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'text',
                    'default'       => '|',
                    'transport'		=> 'postMessage',
                    'active_callback' => array( $this, 'is_register'),
                    'sanitize_callback' => 'sanitize_text_field',
                ),

            ),
        );

        $settings['messages'] = array(
            'title'       => esc_html__( 'Messages', 'clp-custom-login-page' ),
            'description' => '',
            'fields'      => array(
                array(
                    'id'            => 'clp_messages-login_messages-separator',
                    'label'         => esc_html__( 'Login Messages', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'            => 'clp_messages-invalid_username',
                    'label'         => esc_html__( 'Error: Invalid Username', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => __('Unknown username. Check again or try your email address.'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
                array(
                    'id'            => 'clp_messages-invalid_email',
                    'label'         => esc_html__( 'Error: Invalid Email', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => __('Unknown email address. Check again or try your username.'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
                array(
                    'id'            => 'clp_messages-incorrect_password',
                    'label'         => esc_html__( 'Error: Incorrect Password', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => sprintf(__('%1$sError:%2$s The password you entered is incorrect.'), '<strong>', '</strong>'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
                array(
                    'id'            => 'clp_messages-empty_username',
                    'label'         => esc_html__( 'Error: Empty Username', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => sprintf(__('%1$sError:%2$s Please enter a username.'), '<strong>', '</strong>'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
                array(
                    'id'            => 'clp_messages-empty_password',
                    'label'         => esc_html__( 'Error: Empty Password', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => sprintf(__('%1$sError:%2$s The password field is empty'), '<strong>', '</strong>'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
                array(
                    'id'            => 'clp_messages-register-separator',
                    'label'         => esc_html__( 'Register Messages', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'            => 'clp_messages-email_exists',
                    'label'         => esc_html__( 'Error: Email is Already Registered', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => sprintf(__('%1$sError:%2$s This email is already registered, please choose another one.'), '<strong>', '</strong>'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
                array(
                    'id'            => 'clp_messages-username_exists',
                    'label'         => esc_html__( 'Error: Username is Already Registered', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => sprintf(__('%1$sError:%2$s This username is already registered. Please choose another one.'), '<strong>', '</strong>'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
                array(
                    'id'            => 'clp_messages-empty_email',
                    'label'         => esc_html__( 'Error: Empty Email', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => sprintf(__('%1$sError:%2$s Please type your email address.'), '<strong>', '</strong>'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
                array(
                    'id'            => 'clp_messages-register_message',
                    'label'         => esc_html__( 'Info: Register Message', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => __('Register For This Site'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
                array(
                    'id'            => 'clp_messages-register_message2',
                    'label'         => esc_html__( 'Info: Register Message 2', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => __('Registration confirmation will be emailed to you.'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
                array(
                    'id'            => 'clp_messages-lostpassword-separator',
                    'label'         => esc_html__( 'Lost Password Messages', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'separator',
                    'sanitize_callback' => '',
                ),
                array(
                    'id'            => 'clp_messages-invalidcombo',
                    'label'         => esc_html__( 'Error: Account not found', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => sprintf(__('%1$sError:%2$s There is no account with that username or email address.'), '<strong>', '</strong>'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
                array(
                    'id'            => 'clp_messages-forgetpassword_message',
                    'label'         => esc_html__( 'Info: Lost Password Message', 'clp-custom-login-page' ),
                    'description'   => '',
                    'type'          => 'textarea',
                    'default'       => __('Please enter your username or email address. You will receive an email message with instructions on how to reset your password.'),
                    'transport'		=> 'postMessage',
                    'sanitize_callback' => 'wp_filter_post_kses',
                ),
            ),
        );

        $settings['css'] = array(
            'title'       => esc_html__( 'Custom CSS', 'clp-custom-login-page' ),
            'description' => '',
            'fields'      => array(
                array(
                    'id'          => 'clp_css',
                    'label'       => esc_html__( 'Custom CSS', 'clp-custom-login-page' ),
                    'type'        => 'code',
                    'code_type'   => 'text/css',
                    'transport'		=> 'postMessage',
                ),

            ),
        );

        return $settings;
    }

	public function is_background_color( $control ) {
		return 'color' === $control->manager->get_setting( 'clp_background' )->value();
	}

	public function is_background_gradient( $control ) {
		return 'gradient' === $control->manager->get_setting( 'clp_background' )->value();
    }
    
	public function is_background_pattern( $control ) {
		return 'pattern' === $control->manager->get_setting( 'clp_background' )->value();
    }
    

	public function is_custom_pattern( $control ) {
		return 'pattern' === $control->manager->get_setting( 'clp_background' )->value() && 'custom' === $control->manager->get_setting( 'clp_background-pattern' )->value();
    } 
    
	public function is_background_image( $control ) {
		return 'image' === $control->manager->get_setting( 'clp_background' )->value();
    }
    
	public function is_background_video( $control ) {
		return 'video' === $control->manager->get_setting( 'clp_background' )->value();
    }
    
	public function is_logo_text( $control ) {
		return 'text' === $control->manager->get_setting( 'clp_logo' )->value();
	}
	public function is_logo_image( $control ) {
		return 'image' === $control->manager->get_setting( 'clp_logo' )->value();
	}
	public function is_logo_enabled( $control ) {
		return 'image' === $control->manager->get_setting( 'clp_logo' )->value() || 'text' === $control->manager->get_setting( 'clp_logo' )->value();
    }
    
	public function is_overlay_enabled( $control ) {
		return '1' === $control->manager->get_setting( 'clp_background-overlay-enable' )->value();
    }
    
	public function is_form_borders( $control ) {
		return '1' === $control->manager->get_setting( 'clp_form-borders' )->value();
    }

	public function is_form_shadow( $control ) {
		return '1' === $control->manager->get_setting( 'clp_form-shadow' )->value();
    }
    
	public function is_backtoblog( $control ) {
		return '1' === $control->manager->get_setting( 'clp_form_footer-display_backtoblog' )->value();
    }
    
	public function is_forgetpassword( $control ) {
		return '1' === $control->manager->get_setting( 'clp_form_footer-display_forgetpassword' )->value();
    }

    public function is_register( $control ) {
        return '1' === $control->manager->get_setting( 'clp_form_footer-display_register' )->value();
    }

	public function is_rememberme( $control ) {
		return '1' === $control->manager->get_setting( 'clp_inputs-remember' )->value();
    }
    
	public function can_register( $control ) {
		return '1' === get_option( 'users_can_register' );
    }
    
	public function is_privacy_link( $control ) {
		return !empty(get_privacy_policy_url());
    }
    


	public function sanitize_select( $input, $setting ) {
		$input   = sanitize_key( $input );
		$choices = $setting->manager->get_control( $setting->id )->choices;
		return ( array_key_exists( $input, $choices ) ? $input : $setting->default );
    }
    
	public function sanitize_toggle( $input ) {

		return isset( $input ) && true == $input ? '1' : '';
    }
    
	public function sanitize_float( $input ) {
		return filter_var($input, FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
    }

	public function sanitize_negative_int( $input ) {
		return is_numeric( $input ) ? $input : '0';
    }
    
    public function sanitize_alpha_color( $input ) {

        if ( empty( $input ) || is_array( $input ) ) {
            return 'rgba(0,0,0,0)';
        }

        if ( false === strpos( $input, 'rgba' ) ) {
            return sanitize_hex_color( $input );
        }

        $input = str_replace( ' ', '', $input );
        sscanf( $input, 'rgba(%d,%d,%d,%f)', $red, $green, $blue, $alpha );

        return 'rgba('.$red.','.$green.','.$blue.','.$alpha.')';
    }

    private function grab_theme_settings() {
        $default = $this->get_default_layout_settings();

        foreach ( $default as $id => $val) {

            $value = get_option($id);

            if ( !empty($value) && $value !== $val) {
                echo "'". $id  . "' => '" . $value . "',". "<br>";
            }
        }
    }
}