<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
};


class CLP_Helper_Functions {

    /**
     * Returns attachment URL from ID via AJAX
     * @since 1.0.0
    **/
    public static function clp_wp_get_attachment_url_ajax( ) {
        
        // verify user rights
        if( !current_user_can('publish_pages') ) {
            die('Sorry, but this request is invalid');
        }

        $id = '';

        if ( !empty($_REQUEST["id"]) && is_numeric($_REQUEST["id"]) ) {
            $id = filter_var($_REQUEST["id"], FILTER_SANITIZE_NUMBER_INT);
        }

        echo wp_get_attachment_url($id);

        wp_die();
    }

    /**
     * Render customizer link
     * @since 1.0.0
    **/
    public static function customizer_icon_link( $section, $label = 'element', $icon = 'dashicons-edit' ) {
        // bail out early if not in customizer
        if ( !is_customize_preview() ) {
            return;
        }
        
        echo '<span id="'.esc_attr($section).'" class="customize-partial-edit-shortcut clp-customizer-preview" data-section="'.esc_attr($section).'"><button aria-label="Click to edit a '.esc_attr($label).'." title="Click to edit a '.esc_attr($label).'." class="customize-partial-edit-shortcut-button"><span class="dashicons '.$icon.'"></span></button></span>';

        return;
    }

    /**
     * Return Font Variant and Font Weight from Google Font Variant - 300italic => 300, italic
     * @since 1.0.0
     * @return array
    **/
    public static function process_google_font_variant( $variant ) {
        $split = preg_split('#(?<=\d)(?=[a-z])#i', $variant);

        $return = array(
            'weight' => is_numeric($split[0]) ? $split[0] : '400',
            'style' => isset($split[1]) || $split[0] === 'italic' ? 'italic' : 'normal'
        );
        
        return $return;
    }

    /**
     * checks user agent for mobile device
     * @since 1.0.0
     * @return boolean
    **/
    public static function is_mobile() {

        if ( isset($_SERVER["HTTP_USER_AGENT"]) ) {
            return preg_match("/(android|avantgo|blackberry|bolt|boost|cricket|docomo|fone|hiptop|mini|mobi|palm|phone|pie|tablet|up\.browser|up\.link|webos|wos)/i", $_SERVER["HTTP_USER_AGENT"]);

        } else {
            return false;
        }
    }

    /**
     * return image size based on desktop / mobile
     * @since 1.0.0
     * @return boolean
    **/
    public static function get_image_size() {
        return self::is_mobile() ? 'large' : 'full';
    }

}