<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
};

/**
 * Class to render or change HTML elements on login page
 * @since 1.0.0
**/

class CLP_Render_HTML {

    /**
     * Add video scripts to login page footer
     * @since 1.0.0
    **/
    public static function footer_video_script() {

        $video_yt = esc_url( get_option('clp_background-video_yt', '') );
        $video_local = esc_attr( get_option('clp_background-video_local', '') );
        
        
        $source = empty( $video_yt  ) ? empty( $video_local ) ? null : 'video/mp4' : 'YouTube';
        $html = '';

        if ( $source ) {

            $video_autoloop	= get_option('clp_background-video_loop', true);
            $video_poster  = wp_get_attachment_image_src( get_option('clp_background-video_thumb', ''), 'full' );

            if ( !empty( $video_poster ) ) {
                $video_poster = $video_poster[0];       
            }
            
            ob_start();  ?>
            <script type='text/javascript' src='<?php echo CLP_PLUGIN_PATH;?>assets/js/external/vidim.min.js?ver=<?php echo CLP_VERSION;?>'></script>
        
            <script>
                <?php 
                switch ( $source ) {
                    case 'YouTube': ?>
                        var src = '<?php echo $video_yt; ?>'.split('?t=');
                        var myBackground = new vidim( '.login-background', {
                            src: src[0],
                            type: 'YouTube',
                            poster: '<?php echo esc_url( $video_poster ); ?>',
                            quality: 'hd1080',
                            muted: true,
                            loop: <?php echo $video_autoloop ? 'true' : 'false' ; ?>,
                            startAt: src.length > 1 ? src[1] : '0',
                            showPosterBeforePlay: <?php echo $video_poster ? 'true' : 'false' ; ?>
                        });
        
                    <?php 
                        break;
        
                    case 'video/mp4':
                        $video_url = $video_local;
                        $video_url = wp_get_attachment_url( $video_url ); ?>
                        var myBackground = new vidim( '.login-background', {
                            src: [
                                {
                                type: 'video/mp4',
                                src: '<?php echo esc_url( $video_url ); ?>',
                                },
                            ],
                            poster: '<?php echo esc_url( $video_poster ); ?>',
                            showPosterBeforePlay: <?php echo $video_poster ? 'true' : 'false' ; ?>,
                            loop: <?php echo $video_autoloop ? 'true' : 'false' ; ?>
                        });
                        <?php 
                        break;
                    default:
                        break;
                } ?>
            </script>
            <?php

            $html = ob_get_clean();
        }

        return $html;
    }

    /**
     * Add/remove/change logo on login page
     * @since 1.0.0
    **/
    public static function logo() {
        
        $html = '';
        $logo_type = get_option('clp_logo', 'image');

        ob_start();
        
        switch ( $logo_type ) {
            case 'text': ?>
                <h2 class="clp-login-logo">
                    <?php CLP_Helper_Functions::customizer_icon_link( 'clp_logo', 'logo'); ?>
                    <a href="<?php echo esc_html( get_option('clp_logo-url', get_bloginfo('url')) );?>"><?php echo esc_html( get_option('clp_logo-text', get_bloginfo('name')) );?></a>
                </h2>
                <?php 
                break;

            case 'image':
                $logo_url = !get_option('clp_logo-image', '') ? admin_url('images/wordpress-logo.svg?ver=20131107') : wp_get_attachment_url( get_option('clp_logo-image', '') ); ?>
                <h2 class="clp-login-logo">
                    <?php CLP_Helper_Functions::customizer_icon_link( 'clp_logo', 'logo'); ?>
                    <a href="<?php echo esc_html( get_option('clp_logo-url', get_bloginfo('url')) );?>">
                        <img src="<?php echo esc_url( $logo_url );?>" alt="<?php echo esc_html( get_option('clp_logo-text', get_bloginfo('name')) );?> logo">
                    </a>
                </h2>
                <?php
                break;

            case 'none':         
            default:
                break;
        }

        $html = ob_get_clean();

        return $html;
    
    }

    /**
     * Various JavaScript to manipulate HTML on login page
     * @since 1.0.0
    **/
    public static function dom_elements_manipulation() {
        
        $html = '';
        $logo_contained = get_option( 'clp_logo-contained', false );

        ob_start(); ?>

        <script>
            // display logo inside or outside of form container
            const logo = document.querySelector('.clp-login-logo');
            const loginContainer = document.querySelector('<?php echo $logo_contained ? '#login' : '.clp-login-form-container';?>');
            logo ? loginContainer.prepend(logo) : null;
            
            // change back to site text
            const backTo = document.querySelector('#backtoblog > a');
            backTo ? backTo.innerHTML = '<?php echo esc_html( get_option('clp_form_footer-backtoblog_text', sprintf( _x( '&larr; Back to %s', 'site' ), get_bloginfo( 'title', 'display' ) )) );?>' : null;
            
            // fade in whole body
            const body = document.querySelector('body.login');
            body.classList.add('loaded');
        </script>
        
        <?php 
        $html = ob_get_clean();

        return $html;
    
    }
    
}