<?php
// http://justintadlock.com/archives/2015/06/05/javascript-templating-in-the-wordpress-customizer

class CLP_Customizer_Layout_Control extends WP_Customize_Control {

	/**
	 * The control type.
	 *
	 * @access public
	 * @var string
	 */
	public $type = 'clp-layout';


    /**
	 * CLP_Customizer_Layout_Control constructor.
	 *
	 * @param WP_Customize_Manager $manager
	 * @param string               $id
	 * @param array                $args
	 */
	public function __construct( WP_Customize_Manager $manager, $id, array $args = array() ) {
		$manager->register_section_type( 'CLP_Customizer_Layout_Control' );
		parent::__construct( $manager, $id, $args );
	}


	/**
	 * parse parameters to JS JSON
	 *
	 * @access public
	 * @since  1.1.7
	 * @return void
	 */
	public function to_json() {
		parent::to_json();

		$arrays = $this->get_values();

		// The setting value.
		$this->json['id']			= $this->id;
		$this->json['value']		= $this->value();
		$this->json['link']			= $this->get_link();
		$this->json['choices']		= $arrays['choices'];
		$this->json['options']		= $arrays['options'];

	}

	public function get_values() {
        
		$arrays = array(
			'choices' => array(),
			'options' => array(),
		);


		foreach ( $this->choices as $key => $choice ) {

			$arrays['choices'][ $key ]['name'] = $choice['name'];
			$arrays['choices'][ $key ]['thumbnail'] = $choice['thumbnail'];
			$arrays['options'][ $key ] = array();

			foreach ( $choice['options'] as $option_key => $option_value ) {
				$arrays['options'][ $key ][ $option_key ] = array(
					'name' => $option_key,
					'value' => $option_value
				);
			}
		}

		return $arrays;

	}

	/**
	 * Don't render the content via PHP.  This control is handled with a JS template.
	 *
	 * @access public
	 * @since  1.0.0
	 * @return void
	 */
	public function render_content() {}

	/**
	 * An Underscore (JS) template for this control's content.
	 *
	 * Class variables for this control class are available in the `data` JS object;
	 * export custom variables by overriding {@see WP_Customize_Control::to_json()}.
	 *
	 * @see    WP_Customize_Control::print_template()
	 *
	 * @access protected
	 * @since  1.1.7
	 * @return void
	 */
	protected function content_template() {
		?>

		<# if ( ! data.choices ) {
			return;
		} #>
		

		<# if ( data.label ) { #>
			<span class="customize-control-title">{{ data.label }}</span>
		<# } #>


		<# for ( choice in data.choices ) { #>

			<label for="{{ data.id }}-{{ choice }}">
				<input type="radio" value="{{ choice }}" name="_customize-{{ data.id }}" id="{{ data.id }}-{{ choice }}" style="display:none"/>
				<div class="clp-layout-thumbnail">
					<img src="{{ data.choices[choice].thumbnail }}" alt="{{ data.choices[choice].name }}">
					<div class="clp-layout-overlay">
						<span class="clp-layout-overlay-text">{{ data.choices[choice].name }}</span>
					</div>
				</div>
			</label>

		<# } #>

	<?php
	}
}
