<?php 

add_action( 'login_enqueue_scripts', 'clp_login_enqueue_scripts', 10 );
add_action( 'login_enqueue_scripts', 'clp_login_add_custom_css', 999 );
add_action( 'clp_login_header', 'clp_login_add_header_html' );
add_action( 'login_header', 'clp_login_add_header_html' );
add_action( 'clp_login_footer', 'clp_login_add_footer_html' );
add_action( 'login_footer', 'clp_login_add_footer_html' );

add_filter( 'login_errors',	'clp_login_errors' );
add_filter( 'login_body_class', 'clp_login_body_class' );
add_filter( 'gettext', 'clp_lost_password_text', 20, 3 );
add_filter( 'login_link_separator', 'clp_login_link_separator' );

/**
 * Enequeue plugin CSS
 * @since 1.0.0
**/
function clp_login_enqueue_scripts() {
    wp_enqueue_style( 'clp-custom-login', CLP_PLUGIN_PATH . 'assets/css/custom-login.css', array(), CLP_VERSION, 'all');
}

/**
 * Echo custom CSS into login header
 * @since 1.0.0
**/
function clp_login_add_custom_css() {
    // get customizer CSS
    $css = new CLP_Generate_CSS;
    $inline_css = $css->get_customizer_css();

    $rememberme = get_option('clp_inputs-remember', true);
    // form font
    $form_font = json_decode(get_option('clp_form_typography-google_fonts', '{"family":"Roboto","variants":["100","100italic","300","300italic","regular","italic","500","500italic","700","700italic","900","900italic"],"selected":{"variant":"regular"}}'), true);
    $form_variant = CLP_Helper_Functions::process_google_font_variant($form_font['selected']['variant']);

    
    // load google font if logo is text
    if ( get_option('clp_logo', 'image') === 'text' ) { 
        $logo_font = json_decode(get_option('clp_logo-google_fonts', '{"family":"Roboto","variants":["100","100italic","300","300italic","regular","italic","500","500italic","700","700italic","900","900italic"],"selected":{"variant":"regular"}}'), true);
        $variant = CLP_Helper_Functions::process_google_font_variant($logo_font['selected']['variant']);
        wp_enqueue_style( 'logo-google-font', '//fonts.googleapis.com/css?family='. esc_attr( $logo_font['family'] ) .':'.esc_attr( $logo_font['selected']['variant'] ) .'&amp;text='. esc_attr( get_option('clp_logo-text', get_bloginfo('name')) ), array(), '1.0', 'all' );
    }
    
    wp_enqueue_style( 'form-google-font', '//fonts.googleapis.com/css?family='. esc_attr( $form_font['family'] ) .':'.esc_attr( $form_font['selected']['variant'] ), array(), '1.0', 'all' );

    ob_start(); 

    if ( get_option('clp_logo', 'image') === 'text' ) : ?>
    <!-- Logo Google font -->
    <style>
        .login .clp-login-logo {
            font-family: <?php echo esc_attr( $logo_font['family'] ); ?>;
            font-weight: <?php echo esc_attr( $variant['weight'] ); ?>;
            font-style: <?php echo esc_attr( $variant['style'] );?>;
        }
    </style>
    <?php endif; ?>

    <style>
        body.login {
            font-family: <?php echo esc_attr( $form_font['family'] ); ?>;
            opacity: 0;
            transition: opacity 100ms ease-in-out;
        }

        body.login.loaded {
            opacity: 1;
        }

        body.login, .login form, .login form .forgetmenot {
            font-weight: <?php echo esc_attr( $form_variant['weight'] ); ?>;
            font-style: <?php echo esc_attr( $form_variant['style'] );?>;
        }

        .login-overlay {
            background-color: rgba(0,0,0,0.4);
        }
        
        <?php 
        // output customizer CSS
        echo $inline_css;
        ?>
    </style>

    <style id="clp-custom-css">
        <?php echo get_option('clp_css', '') !== '' ? get_option('clp_css', '') : null; ?>
    </style>
    <?php
    echo ob_get_clean();
}

/**
 * Echo custom HTML into login page
 * @since 1.0.0
**/
function clp_login_add_header_html() { 
    ?>
    <div class="clp-background-wrapper">
        <div class="login-background"></div>
    </div>

    <div class="login-overlay" style="display:<?php echo get_option('clp_background-overlay-enable', '') === ''  ? 'none' : 'block';?>"></div>
    <div class="clp-form-container">
        <div class="clp-login-form-container">
            
    
    <?php 

    echo CLP_Render_HTML::logo();
}



/**
 * Echo custom HTML into login page footer
 * @since 1.0.0
**/
function clp_login_add_footer_html() {
    ?>
    <!-- class="clp-login-form-container" -->
    </div>
    <!-- class="clp-form-container" -->
    </div> 
    <?php
    echo CLP_Render_HTML::dom_elements_manipulation();
    
    echo get_option('clp_background', 'color') === 'video' ? CLP_Render_HTML::footer_video_script() : null;

}

/**
 * Add specific classes to body tag
 * @since 1.0.0
**/
function clp_login_body_class( $classes  ) { 
    $classes[] = 'clp-login';
    $classes[] = 'clp-' . get_option('clp_logo', 'image') . '-logo';
    $classes[] = 'clp-background-' . get_option('clp_background', 'image');
    $classes[] = get_option('clp_background', 'image') === 'pattern' ? 'clp-pattern-' . get_option('clp_background-pattern', 'fabric') : null;
    $classes[] = get_option('clp_background-blur', '0') ? 'clp-background-blur' : '';
    $classes[] = !get_option('clp_inputs-remember', true) ? 'clp-hide-rememberme' : '';
    $classes[] = !get_option('clp_inputs-showpassword', true) ? 'clp-hide-show-pw' : '';
    $classes[] = !get_option('clp_form_footer-display_backtoblog', '1') ? 'clp-hide-backtoblog' : '';
    $classes[] = !get_option('clp_form_footer-display_forgetpassword', '1') ? 'clp-hide-forgetpassword' : '';
    $classes[] = !get_option('clp_form_footer-display_register', '1') ? 'clp-hide-register' : '';
    $classes[] = 'form-footer-' . get_option('clp_form_footer-align', 'left');
    $classes[] = 'can-register-' . get_option('users_can_register');
    
	return $classes;
}

/**
 * Change login error meessages
 * @since 1.0.0
**/
function clp_login_errors( $error ) { 
    global $errors;
    

    $err_codes = $errors->get_error_codes();

    // Invalid username.
	if ( in_array( 'invalid_username', $err_codes ) ) {
		$error = get_option( 'clp_messages-invalid_username') ? get_option( 'clp_messages-invalid_username') : $error;
    }
    // Invalid email.
	if ( in_array( 'invalid_email', $err_codes ) ) {
		$error = get_option( 'clp_messages-invalid_email' ) ? get_option( 'clp_messages-invalid_email' ) : $error;
    }
    // Empty Username
	if ( in_array( 'empty_username', $err_codes ) ) {
		$error = get_option( 'clp_messages-empty_username' ) ? get_option( 'clp_messages-empty_username' ) : $error;
    }
    // Empty Password
	if ( in_array( 'empty_password', $err_codes ) ) {
		$error = get_option( 'clp_messages-empty_password' ) ? get_option( 'clp_messages-empty_password' ) : $error;
    }
    // Incorrect password.
	if ( in_array( 'incorrect_password', $err_codes ) ) {
        $error = get_option( 'clp_messages-incorrect_password' ) ? get_option( 'clp_messages-incorrect_password' ) : $error;
    }
    // Account not found to get password
	if ( in_array( 'invalidcombo', $err_codes ) ) {
        $error = get_option( 'clp_messages-invalidcombo' ) ? get_option( 'clp_messages-invalidcombo' ) : $error;
    }
    // Username Taken
	if ( in_array( 'username_exists', $err_codes ) ) {
        $error = get_option( 'clp_messages-username_exists' ) ? get_option( 'clp_messages-username_exists' ) : $error;
    }
    // Email Taken
	if ( in_array( 'email_exists', $err_codes ) ) {
        $error = get_option( 'clp_messages-email_exists' ) ? get_option( 'clp_messages-email_exists' ) : $error;
    }
    // Empty Email
	if ( in_array( 'empty_email', $err_codes ) ) {
        $error = get_option( 'clp_messages-empty_email' ) ? get_option( 'clp_messages-empty_email' ) : $error;
    }
    
	return $error;
}

/**
 * Customize form footer separator text
 * @since 1.0.0
**/
function clp_login_link_separator( $login_link_separator ) {
    return !get_option('clp_form_footer-display_register', '1') ? '' : ' ' . get_option('clp_form_footer-login_link_separator', '|') . ' ';
}

/**
 * Customize form input texts
 * @since 1.0.0
**/
function clp_lost_password_text ( $translation, $text, $domain ) {

    switch ( $text ) {
        case 'Remember Me':
            $translation = get_option('clp_inputs-remember_text') ? get_option('clp_inputs-remember_text') : $translation;
            break;
        case 'Lost your password?':
            $translation = get_option('clp_form_footer-forgetpassword_text') ? get_option('clp_form_footer-forgetpassword_text') : $translation;
            break;
        case 'Log in':
            $translation = get_option('clp_form_footer-login_text') ? get_option('clp_form_footer-login_text') : $translation;
            break;
        // login button
        case 'Log In':
            $translation = get_option('clp_button-text') ? get_option('clp_button-text') : $translation;
            break;
        // register text
        case 'Register':
            $translation = get_option('clp_form_footer-register_text') ? get_option('clp_form_footer-register_text') : $translation;
            break;        
        case 'Register For This Site':
            $translation = get_option('clp_messages-register_message') ? get_option('clp_messages-register_message') : $translation;
            break;        
        case 'Registration confirmation will be emailed to you.':
            $translation = get_option('clp_messages-register_message2') ? get_option('clp_messages-register_message2') : $translation;
            break;
        // Lost Password Message
        case 'Please enter your username or email address. You will receive an email message with instructions on how to reset your password.':
            $translation = get_option('clp_messages-forgetpassword_message') ? get_option('clp_messages-forgetpassword_message') : $translation;
            break;    
        default:
            break;
    }


    return $translation;
}